Index: res/res_srtp.c
===================================================================
--- res/res_srtp.c	(revision 377918)
+++ res/res_srtp.c	(working copy)
@@ -327,6 +327,9 @@
 
 tryagain:
 
+	res = err_status_replay_old;
+	goto fish;
+
 	for (i = 0; i < 2; i++) {
 		res = rtcp ? srtp_unprotect_rtcp(srtp->session, buf, len) : srtp_unprotect(srtp->session, buf, len);
 		if (res != err_status_no_ctx) {
@@ -345,6 +348,8 @@
 		}
 	}
 
+fish:
+
 	if (retry == 0  && res == err_status_replay_old) {
 		ast_log(AST_LOG_NOTICE, "SRTP unprotect failed with %s, retrying\n", srtp_errstr(res));
 
@@ -366,7 +371,7 @@
 
 			ast_debug(5, "SRTP try to re-create\n");
 			if (policy) {
-				if (srtp_create(&srtp->session, &policy->sp) == err_status_ok) {
+				if (srtp_create(&srtp->session, &policy->sp) != err_status_ok) {
 					ast_debug(5, "SRTP re-created with first policy\n");
 					ao2_t_ref(policy, -1, "Unreffing first policy for re-creating srtp session");
 
@@ -383,15 +388,22 @@
 					retry++;
 					ao2_iterator_destroy(&it);
 					goto tryagain;
-				} else {
-					srtp->session = NULL;
 				}
+
+				ast_log(LOG_NOTICE, "Forcefully setting the session to NULL. This should cause the call to hangup.\n");
+				srtp->session = NULL;
 				ao2_t_ref(policy, -1, "Unreffing first policy after srtp_create failed");
 			}
 			ao2_iterator_destroy(&it);
 		}
 	}
 
+	if (!srtp->session) {
+		ast_log(LOG_ERROR, "SRTP session was destroyed and could not be recovered.\n");
+		errno = EINVAL;
+		return -1;
+	}
+
 	if (res != err_status_ok && res != err_status_replay_fail ) {
 		if ((srtp->warned >= 10) && !((srtp->warned - 10) % 100)) {
 			ast_log(AST_LOG_WARNING, "SRTP unprotect failed with: %s %d\n", srtp_errstr(res), srtp->warned);
Index: res/res_rtp_asterisk.c
===================================================================
--- res/res_rtp_asterisk.c	(revision 377918)
+++ res/res_rtp_asterisk.c	(working copy)
@@ -355,6 +355,10 @@
 	return 1;
 }
 
+static int times_displayed;
+static int skips;
+static struct ast_srtp *last_srtp;
+
 static int __rtp_recvfrom(struct ast_rtp_instance *instance, void *buf, size_t size, int flags, struct ast_sockaddr *sa, int rtcp)
 {
 	int len;
@@ -365,6 +369,25 @@
 	   return len;
 	}
 
+	if (res_srtp && srtp) {
+		if (last_srtp != srtp) {
+			last_srtp = srtp;
+			times_displayed = -5;
+			skips = 0;
+		}
+
+		if (skips >= times_displayed) {
+			ast_log(LOG_NOTICE, "We are going to run unprotect on %p\n", srtp);
+			times_displayed++;
+			if (times_displayed > 0) {
+				times_displayed += times_displayed / 2;
+			}
+			skips = 0;
+		} else {
+			skips++;
+		}
+	}
+
 	if (res_srtp && srtp && res_srtp->unprotect(srtp, buf, &len, rtcp) < 0) {
 	   return -1;
 	}
@@ -2142,15 +2165,19 @@
 		ast_rtp_dtmf_continuation(instance);
 	}
 
+	errno = 0;
+
 	/* Actually read in the data from the socket */
 	if ((res = rtp_recvfrom(instance, rtp->rawdata + AST_FRIENDLY_OFFSET,
 				sizeof(rtp->rawdata) - AST_FRIENDLY_OFFSET, 0,
 				&addr)) < 0) {
+		ast_log(LOG_NOTICE, "errno = %s\n", strerror(errno));
 		ast_assert(errno != EBADF);
 		if (errno != EAGAIN) {
 			ast_log(LOG_WARNING, "RTP Read error: %s. Hanging up.\n", strerror(errno));
 			return NULL;
 		}
+		ast_log(LOG_NOTICE, "poke EAGAIN\n");
 		return &ast_null_frame;
 	}
 

